import pandas as pd
import numpy as np
import itertools
from functools import reduce
import os
import multiprocessing
import time
import warnings
from tqdm import tqdm

warnings.filterwarnings("ignore")

pd.options.display.width = 1200
pd.options.display.max_colwidth = 100
pd.options.display.max_columns = 10
pd.options.mode.chained_assignment = None


def level10_Diff(df, lag=20):
    """
    十档委买增额
    :param df:
    :param lag:
    :return:
    """
    temp = df[["SecurityID", "TradeTime"]]

    for i in range(10):
        temp[f"bid{i+1}"] = df[f"BidPrice{i+1}"].fillna(0)
        temp[f"bidAmt{i+1}"] = df[f"BidOrderQty{i+1}"].fillna(0) * df[
            f"BidPrice{i+1}"
        ].fillna(0)
        temp[f"prevbid{i+1}"] = temp[f"bid{i+1}"].shift(1).fillna(0)
        temp[f"prevbidAmt{i+1}"] = temp[f"bidAmt{i+1}"].shift(1).fillna(0)

    temp["bidMin"] = temp[[f"bid{i+1}" for i in range(10)]].min(axis=1)
    temp["bidMax"] = temp[[f"bid{i+1}" for i in range(10)]].max(axis=1)
    temp["prevbidMin"] = temp[[f"prevbid{i+1}" for i in range(10)]].min(axis=1)
    temp["prevbidMax"] = temp[[f"prevbid{i+1}" for i in range(10)]].max(axis=1)
    temp["pmin"] = temp[["bidMin", "prevbidMin"]].max(axis=1)
    temp["pmax"] = temp[["bidMax", "prevbidMax"]].max(axis=1)

    temp["amtDiff"] = 0.0
    for i in range(10):
        temp["amtDiff"] += temp[f"bidAmt{i+1}"] * (
            (temp[f"bid{i+1}"] >= temp["pmin"]) & (temp[f"bid{i+1}"] <= temp["pmax"])
        ).astype(int) - temp[f"prevbidAmt{i+1}"] * (
            (temp[f"prevbid{i+1}"] >= temp["pmin"])
            & (temp[f"prevbid{i+1}"] <= temp["pmax"])
        ).astype(
            int
        )
    temp["amtDiff"] = temp.groupby("SecurityID")["amtDiff"].apply(
        lambda x: x.rolling(lag, 1).sum()
    )
    return temp[["SecurityID", "TradeTime", "amtDiff"]].fillna(0)


def pool_func(tick_obj, snapshot_path_obj):
    single_tick_res = []
    for tick in tqdm(tick_obj):
        try:
            df = pd.read_hdf(os.path.join(snapshot_path_obj, tick))
            single_tick_res.append(level10_Diff(df, lag=20))
        except Exception as error:
            single_tick_res = pd.DataFrame(
                columns=["SecurityID", "TradeTime", "amtDiff"]
            )
            continue
    return pd.concat(single_tick_res, axis=0)


class multi_task_split:
    def __init__(self, data, processes_to_use):
        self.data = data
        self.processes_to_use = processes_to_use

    def num_of_jobs(self):
        return min(len(self.data), self.processes_to_use, multiprocessing.cpu_count())

    def split_args(self):
        q, r = divmod(len(self.data), self.num_of_jobs())
        return (
            self.data[i * q + min(i, r) : (i + 1) * q + min(i + 1, r)]
            for i in range(self.num_of_jobs())
        )


if __name__ == "__main__":
    n_use = 16
    # 路径修改为存放数据路径
    snapshot_path = "/home/wjchen/workfile/python_plus_file_VS_dolphindb_on_factor_calc/hdf5File"
    stock_pool = os.listdir(snapshot_path)
    processes_decided = multi_task_split(stock_pool, n_use).num_of_jobs()
    print(processes_decided)
    split_args_to_process = list(multi_task_split(stock_pool, n_use).split_args())
    args = [
        (split_args_to_process[i], snapshot_path)
        for i in range(len(split_args_to_process))
    ]
    print("#" * 50 + "Multiprocessing Start" + "#" * 50)
    t0 = time.time()
    with multiprocessing.Pool(processes=processes_decided) as pool:
        res = tqdm(pool.starmap(pool_func, args))
        print("cal time: ", time.time() - t0, "s")
        res_combined = pd.concat(res, axis=0)
        pool.close()
        print("cal time: ", time.time() - t0, "s")
    print(res_combined)
